<?php

// If this file is called directly, abort.
if ( ! defined('ABSPATH')) {
    die;
}

/**
 * Class MP_Locations
 *
 * @author Dominykas Tijūnaitis (dominykas@tijunaitis.lt)
 */
class MP_Locations
{
    const TYPE_TERMINAL = 'terminal';
    const TYPE_PICKUP_POINT = 'pickup_point';

    /** @var string */
    private $table = 'multiparcels_terminals';

    /**
     * @param string|null $carrier_code
     * @param string|null $country
     *
     * @return mixed
     */
    public function all($carrier_code = null, $country = null)
    {
        global $wpdb;

        if ($carrier_code && $country) {
            $query = $wpdb->prepare("SELECT * FROM " . $this->table() . " WHERE `country_code` = '%s' AND `carrier_code` = %s",
                $country, $carrier_code);
        } elseif ($carrier_code) {
            $query = $wpdb->prepare("SELECT * FROM " . $this->table() . " WHERE `carrier_code` = %s", $carrier_code);
        } else {
            $query = "SELECT * FROM " . $this->table();
        }

        return $wpdb->get_results($query, ARRAY_A);
    }

    /**
     * @param string $identifier
     * @param string $carrier_code
     *
     * @return array
     */
    public function get($identifier, $carrier_code)
    {
        global $wpdb;

        $query = $wpdb->prepare("SELECT * FROM " . $this->table() . " WHERE `identifier` = %s AND `carrier_code` = %s",
            $identifier, $carrier_code);

        return $wpdb->get_row($query, ARRAY_A);
    }

    public function grouped_by_city($carrier_code, $country)
    {
        $groups    = [];
        $locations = $this->all($carrier_code, $country);

        foreach ($locations as $location) {
            $city                        = $this->latin_only($location['city']);
            $location['city_simplified'] = $city;

            $groups[$city][] = $location;
        }

        ksort($groups);

        return $groups;
    }

    public function get_for_city($carrier_code, $city, $country)
    {
        $city   = $this->latin_only($city);
        $groups = $this->grouped_by_city($carrier_code, $country);

        if (isset($groups[$city])) {
            return [
                $city => $groups[$city],
            ];
        }

        return $this->grouped_by_city($carrier_code, $country);
    }

    private function latin_only($text)
    {
        // transliterate
        $text = iconv('utf-8', 'us-ascii//TRANSLIT', $text);

        // remove unwanted characters
        $text = preg_replace('~[^-\w]+~', '', $text);

        // lowercase
        $text = strtolower($text);

        if (empty($text)) {
            return 'n-a';
        }

        return $text;
    }

    /**
     * Returns prefixed table name
     *
     * @return string
     */
    private function table()
    {
        global $wpdb;

        return $wpdb->prefix . $this->table;
    }

    /**
     * Create a location
     *
     * @param array $data
     */
    public function create($data)
    {
        global $wpdb;

        $wpdb->insert($this->table(), $data);
    }

    public function clear()
    {
        global $wpdb;

        $wpdb->query("DELETE FROM " . $this->table());
    }

    /**
     * @param string $type
     *
     * @return string
     */
    public function type_name($type)
    {
        if ($type == self::TYPE_TERMINAL) {
            return __('Terminal', 'multiparcels-shipping-for-woocommerce');
        }
        if ($type == self::TYPE_PICKUP_POINT) {
            return __('Pickup Point', 'multiparcels-shipping-for-woocommerce');
        }

        return $type;
    }

    /**
     * @param array $location
     *
     * @return string
     */
    public function selected_text($location)
    {
        $selected_text = __("Selected terminal", 'multiparcels-shipping-for-woocommerce');

        if ($location['type'] == MP_Locations::TYPE_PICKUP_POINT) {
            $selected_text = __("Selected pickup point", 'multiparcels-shipping-for-woocommerce');
        }

        return $selected_text;
    }

    public function update()
    {
        $this->clear();

        $types = [
            self::TYPE_TERMINAL,
            self::TYPE_PICKUP_POINT,
        ];

        // We only support pickup locations for these countries
        $countries = [
            'LT',
            'LV',
            'EE',
        ];

        foreach ($countries as $country) {

            foreach ($types as $type) {
                $search   = [];
                $search[] = 'country_code:' . $country;
                $search[] = 'type:' . $type;

                $response = MultiParcels()->api_client->request('locations', 'GET', [
                    'limit'      => 9999,
                    'search'     => implode(';', $search),
                    'searchJoin' => 'and',
                ]);

                if ($response->was_successful()) {
                    $locations = $response->get_data();

                    if (count($locations) > 0) {
                        foreach ($locations as $locationData) {
                            $location = [
                                'carrier_code' => $locationData['courier_code'],
                                'type'         => $locationData['type'],
                                'name'         => $locationData['name'],
                                'address'      => $locationData['address'],
                                'postal_code'  => $locationData['postal_code'],
                                'city'         => $locationData['city'],
                                'country_code' => $locationData['country_code'],
                                'identifier'   => $locationData['identifier'],
                            ];

                            MultiParcels()->locations->create($location);
                        }

                        MultiParcels()->options->set('last_update', current_time('Y-m-d H:i:s'));
                    }
                }
            }
        }
    }

    /**
     * @param int|WC_Order $order
     * @param WC_MP_Shipping_Method|null $shipping
     * @param string|null $courier_code
     *
     * @return array|null
     */
    public function get_location_for_order($order, $shipping = null, $courier_code = null)
    {
        if (is_numeric($order)) {
            $order = wc_get_order($order);
        }

        $allowed = false;

        if ($shipping == null) {
            $allowed = true;
        }

        if ($shipping != null && $order->has_shipping_method($shipping->id)) {
            $courier_code = $shipping->carrier_code;
            $allowed      = true;
        }

        if ($allowed) {
            if (version_compare(WC_VERSION, '3.0.0', '>=')) {
                $order_id = $order->get_id();
            } else {
                $order_id = $order->id;
            }

            $location_identifier = get_post_meta($order_id, 'multiparcels_location_identifier', true);
            $location            = MultiParcels()->locations->get($location_identifier, $courier_code);

            if ($location) {
                return $location;
            }
        }

        return null;
    }

}

return new MP_Locations();
