<?php

// If this file is called directly, abort.
if ( ! defined('ABSPATH')) {
    die;
}

if (class_exists('WC_Shipping_Method')) {

    /**
     * Class WC_MP_Shipping_Method
     *
     * @author Dominykas Tijūnaitis (dominykas@tijunaitis.lt)
     */
    class WC_MP_Shipping_Method extends WC_Shipping_Method
    {
        const TYPE_TERMINALS_ONLY = 'terminals_only';
        const TYPE_TERMINALS_AND_PICKUP_POINTS = 'terminals_and_pickup_points';
        const TYPE_PICKUP_POINTS_ONLY = 'pickup_points';

        /** @var string */
        private $input_name = 'multiparcels_location_identifier';

        /** @var string */
        public $carrier_code;

        /** @var string */
        protected $courier_name;

        /** @var string */
        protected $type_title;

        /** @var bool */
        protected $type = self::TYPE_TERMINALS_ONLY;

        /** @var array */
        private $courier_settings;

        /** @var bool */
        private $courier_service;

        /**
         * Constructor for your shipping class
         *
         * @access public
         *
         * @param string $carrier_code
         * @param array $courier_settings
         * @param bool $courier_service
         */
        public function __construct($carrier_code, $courier_settings, $courier_service = false)
        {
            $this->courier_service  = $courier_service;
            $this->courier_settings = $courier_settings;
            $this->carrier_code     = $carrier_code;

            $id_suffix = 'pickup_points';

            if ($courier_service) {
                $id_suffix = 'courier';
            }

            $this->courier_name = $this->courier_settings['name'];
            $this->id           = sprintf('multiparcels_%s_%s', $this->carrier_code, $id_suffix);

            $this->type_title         = $this->prepare_type_title();
            $this->method_description = $this->prepare_method_description();
            $this->method_title       = sprintf('%s %s', $this->courier_name, $this->type_title);
            $this->title              = sprintf('%s %s', $this->courier_name, $this->type_title);
            $this->enabled            = $this->get_option('enabled', 'no');
            $this->supports           = array(
                'instance-settings',
                'instance-settings-modal',
                'settings',
            );
            $this->init();

            parent::__construct();
        }

        /**
         * @return string
         */
        function prepare_type_title()
        {
            if ($this->courier_service) {
                $this->type_title = __("Courier", 'multiparcels-shipping-for-woocommerce');

                return $this->type_title;
            }
            $this->type_title = __('Terminals', 'multiparcels-shipping-for-woocommerce');

            if ($this->courier_settings['has_pickup_points']) {
                $this->type_title = __('Terminals/Pickup Points', 'multiparcels-shipping-for-woocommerce');

                if ($this->courier_settings['has_terminals'] == false) {
                    $this->type_title = __('Pickup Points', 'multiparcels-shipping-for-woocommerce');
                }
            }

            return $this->type_title;
        }

        /**
         * @param WC_Order|int $order
         */
        function admin_order_data_after_shipping_address($order)
        {
            if ($location = MultiParcels()->locations->get_location_for_order($order, $this)) {
                $selected_text = MultiParcels()->locations->selected_text($location);

                $output = '<div>';
                $output .= sprintf('<div><strong>%s:</strong></div>', esc_html($selected_text));
                $output .= $location['name'] . '<br/>';
                $output .= esc_html(sprintf("%s, %s, %s", $location['address'], $location['city'],
                    $location['postal_code']));
                $output .= '</div>';
                $output .= '<br/>';

                echo $output;
            }
        }

        public function after_checkout_v2($order)
        {
            if ($location = MultiParcels()->locations->get_location_for_order($order, $this)) {
                $selected_text = MultiParcels()->locations->selected_text($location);

                wc_get_template('order/order-details-after-customer-v2.php', [
                    'selected_text' => $selected_text,
                    'location'      => $location['name'],
                    'address'       => sprintf("%s, %s, %s", $location['address'],
                        $location['city'],
                        $location['postal_code']),
                ]);
            }
        }

        public function after_checkout_v3($order)
        {
            if ($location = MultiParcels()->locations->get_location_for_order($order, $this)) {
                $selected_text = MultiParcels()->locations->selected_text($location);

                wc_get_template('order/order-details-after-customer-v3.php', [
                    'selected_text' => $selected_text,
                    'location'      => $location['name'],
                    'address'       => sprintf("%s, %s, %s", $location['address'],
                        $location['city'],
                        $location['postal_code']),
                ]);
            }
        }

        public function email_customer_details($order)
        {
            if ($location = MultiParcels()->locations->get_location_for_order($order, $this)) {
                $selected_text = MultiParcels()->locations->selected_text($location);

                wc_get_template('emails/email-selected-pickup-location.php', [
                    'selected_text' => $selected_text,
                    'location'      => $location['name'],
                    'address'       => sprintf("%s, %s, %s", $location['address'],
                        $location['city'],
                        $location['postal_code']),
                ]);
            }
        }

        function checkout_save_order_terminal_id_meta($order_id, $posted)
        {
            if (isset($_POST[$this->input_name])) {
                $input = sanitize_text_field($_POST[$this->input_name]);
                update_post_meta($order_id, $this->input_name, $input);
            }
        }

        /**
         * @param WC_Shipping_Rate $method
         * @param int $index
         */
        function display_location_select($method, $index)
        {
            $selected_methods = WC()->session->get('chosen_shipping_methods');
            $selected         = count($selected_methods) && in_array($this->id, $selected_methods);
            if ($method->id == $this->id) {
                if ($selected) {
                    if ( ! is_cart()) {
                        $country = null;
                        $city    = null;

                        if (isset($_POST['city'])) {
                            $city = $_POST['city'];
                        }

                        if (isset($_POST['country'])) {
                            $country = $_POST['country'];
                        }

                        $city    = sanitize_text_field($city);
                        $country = sanitize_text_field($country);

                        if (isset($_POST['s_country']) && $_POST['s_country'] != $country) {
                            $country = $_POST['s_country'];
                        }

                        $data = [
                            'id'                      => 'multiparcels-select-id',
                            'name'                    => $this->input_name,
                            'location_groups_by_city' => MultiParcels()->locations->get_for_city($this->carrier_code,
                                $city, $country),
                        ];

                        wc_get_template('checkout/form-shipping-pickup-location-select.php', $data);
                    }
                }
            }
        }

        function init_form_fields()
        {
            $this->form_fields = [
                'enabled'                      => [
                    'title'   => __('Enable/Disable', 'multiparcels-shipping-for-woocommerce'),
                    'type'    => 'checkbox',
                    'label'   => __('Enable', 'multiparcels-shipping-for-woocommerce'),
                    'default' => 'no',
                ],
                'title'                        => [
                    'title'       => __('Title', 'multiparcels-shipping-for-woocommerce'),
                    'type'        => 'text',
                    'description' => __('This controls the title which the user sees during checkout.',
                        'multiparcels-shipping-for-woocommerce'),
                    'default'     => $this->courier_name . ' ' . $this->type_title,
                    'desc_tip'    => true,
                ],
                'fee'                          => [
                    'title'       => __('Delivery Fee', 'multiparcels-shipping-for-woocommerce'),
                    'type'        => 'price',
                    'description' => '',
                    'default'     => '',
                    'desc_tip'    => true,
                    'placeholder' => wc_format_localized_price(0),
                ],
                'min_amount_for_free_shipping' => [
                    'title'       => __('Minimum Order Amount For Free Shipping',
                        'multiparcels-shipping-for-woocommerce'),
                    'type'        => 'price',
                    'placeholder' => wc_format_localized_price(0),
                    'description' => __('Users will need to spend this amount to get free shipping (if enabled above).',
                        'multiparcels-shipping-for-woocommerce'),
                    'default'     => '0',
                    'desc_tip'    => true,
                ],
            ];
        }

        /**
         * calculate_shipping function.
         *
         * @access public
         *
         * @param mixed $package
         *
         * @return void
         */
        public function calculate_shipping($package = [])
        {
            $rate = [
                'id'    => $this->id,
                'label' => $this->get_option('title'),
                'cost'  => $this->get_option('fee'),
            ];

            $this->free_shipping_check($package, $rate);

            $this->add_rate($rate);
        }

        /**
         * @param array $package
         *
         * @return bool
         */
        public function is_available($package)
        {
            if ('no' == $this->enabled) {
                return false;
            }

            // Pickup locations
            if ($this->courier_service == false) {
                $ship_to_countries = $this->courier_settings['pickup_points_countries'];

                if (is_array($ship_to_countries) &&
                    ! in_array($package['destination']['country'], $ship_to_countries)) {
                    return false;
                }
            }

            return true;
        }

        /**
         * @param array $package
         * @param array $rate
         */
        private function free_shipping_check($package, &$rate)
        {
            $order_cost                   = 0;
            $min_amount_for_free_shipping = intval($this->get_option('min_amount_for_free_shipping'));

            if ($min_amount_for_free_shipping > 0) {
                if (array_key_exists('contents_cost', $package)) {
                    $order_cost = intval($package['contents_cost']);
                }

                if ($order_cost >= $min_amount_for_free_shipping) {
                    $rate['cost'] = 0;
                }
            }
        }

        /**
         * @param array $posted
         */
        function check_for_selected_location($posted)
        {
            if (isset($_POST[$this->input_name]) && $_POST[$this->input_name] == '') {
                if (isset($posted['shipping_method']) && is_array($posted['shipping_method'])) {
                    if (in_array($this->id, $posted['shipping_method'])) {
                        wc_add_notice(__('Please select the pickup location.', 'multiparcels-shipping-for-woocommerce'),
                            'error');
                    }
                }
            }
        }

        /**
         * Init your settings
         *
         * @return void
         */
        function init()
        {
            add_action('woocommerce_update_options_shipping_' . $this->id, [$this, 'process_admin_options']);
            $this->init_form_fields();
            $this->init_settings();

            if ($this->courier_service == false) {
                add_action('woocommerce_after_shipping_rate', [$this, 'display_location_select'], 2, 2);
                add_action('woocommerce_after_checkout_validation', [$this, 'check_for_selected_location'], 10, 1);
                add_action('woocommerce_checkout_update_order_meta', [$this, 'checkout_save_order_terminal_id_meta'],
                    10, 2);

                // Places to show the selected location at
                add_action('woocommerce_email_customer_details', [$this, 'email_customer_details'], 15, 1); // email

                if (version_compare(WC_VERSION, '3.0.0', '>=')) {
                    add_action('woocommerce_order_details_after_order_table', [$this, 'after_checkout_v3'], 10);
                } else {
                    add_action('woocommerce_order_details_after_customer_details', [$this, 'after_checkout_v2'], 15, 1);
                }

                if (is_admin()) {
                    add_action('woocommerce_admin_order_data_after_shipping_address',
                        [$this, 'admin_order_data_after_shipping_address'], 20); // Admin orders
                }
            }

            add_action('woocommerce_email_before_order_table', [$this, 'email_instructions'], 10, 4);
        }

        /**
         * @param WC_Order $order
         * @param $sent_to_admin
         * @param bool $plain_text
         * @param WC_Email $email
         */
        public function email_instructions($order, $sent_to_admin, $plain_text = false, $email)
        {
            // Only on completed order
            if ($email->id == 'customer_completed_order' && $order->has_shipping_method($this->id) && ! $sent_to_admin) {
                $tracking_code = get_post_meta($order->id, 'multiparcels_tracking_code', true);

                if ( ! $plain_text && $tracking_code) {
                    $tracking_code = sprintf("<strong>%s</strong>", $tracking_code);
                }

                if ($tracking_code) {
                    $text = __('Your order tracking number: %s', 'multiparcels-shipping-for-woocommerce');
                    echo wpautop(wptexturize(sprintf($text, $tracking_code))) . PHP_EOL;
                }
            }
        }

        /**
         * @return string
         */
        private function prepare_method_description()
        {
            $text = '';

            if ($this->courier_service) {
                $text = sprintf(__("%s courier will deliver the parcel right to the customer's hands.",
                    'multiparcels-shipping-for-woocommerce'), MultiParcels()->carriers->name($this->carrier_code));

                if ($this->courier_settings['cod_allowed']) {
                    $text .= __(' This courier allows cash-on-delivery service.',
                        'multiparcels-shipping-for-woocommerce');
                }
            } else {
                if ($this->courier_settings['has_terminals']) {
                    if ($this->courier_settings['cod_allowed']) {
                        $text = __(' This courier allows cash-on-delivery service when delivering to self-service parcel terminals.',
                            'multiparcels-shipping-for-woocommerce');
                    }
                }
            }

            return $text;
        }
    }
}